use std::fmt;

pub enum FieldType {
    StaticString(StaticString),
}

impl fmt::Display for FieldType {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            FieldType::StaticString(data) => write!(f, "{}", data),
        }
    }
}

impl From<StaticString> for FieldType {
    fn from(data: StaticString) -> Self {
        FieldType::StaticString(data)
    }
}

pub struct StaticString {
    data: String,
}

impl StaticString {
    pub fn new<S>(name: S) -> Self
    where
        S: Into<String>,
    {
        Self { data: name.into() }
    }
}

impl fmt::Display for StaticString {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{}", &self.data)
    }
}

#[cfg(test)]
mod fieldtypes {
    use super::*;

    #[test]
    fn convert_static_string() {
        let data = "a static string";
        let field = StaticString::new(data);
        let ftype: FieldType = field.into();
        assert!(
            ftype.to_string() == data,
            "\n\nGot:  {}\nWant: {}",
            ftype.to_string(),
            data
        );
    }
}

#[cfg(test)]
mod staticstrings {
    use super::*;

    #[test]
    fn create_static_string() {
        let data = "some data";
        let field = StaticString::new(data);
        assert!(
            field.to_string() == data,
            "\n\nGot:  {}\nWant: {}",
            field.to_string(),
            data
        );
    }

    #[test]
    fn accepts_string() {
        let data = "actual string";
        let field = StaticString::new(data.to_string());
        assert!(
            field.to_string() == data,
            "\n\nGot:  {}\nWant: {}",
            field.to_string(),
            data
        );
    }
}
